@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, 
+ * Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 
@@ -25,8 +25,8 @@
 #include "gitg-preferences.h"
 #include "gitg-data-binding.h"
 #include "gitg-utils.h"
-#include "gitg-config.h"
 
+#include <libgitg/gitg-config.h>
 #include <stdlib.h>
 #include <glib/gi18n.h>
 
@@ -37,6 +37,18 @@ enum
 	N_COLUMNS
 };
 
+void on_collapse_inactive_lanes_changed (GtkAdjustment *adjustment,
+                                         GParamSpec *spec,
+                                         GitgPreferencesDialog *dialog);
+
+gboolean on_entry_configuration_user_name_focus_out_event (GtkEntry *entry,
+                                                           GdkEventFocus *event,
+                                                           GitgPreferencesDialog *dialog);
+
+gboolean on_entry_configuration_user_email_focus_out_event (GtkEntry *entry,
+                                                            GdkEventFocus *event,
+                                                            GitgPreferencesDialog *dialog);
+
 #define GITG_PREFERENCES_DIALOG_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE((object), GITG_TYPE_PREFERENCES_DIALOG, GitgPreferencesDialogPrivate))
 
 static GitgPreferencesDialog *preferences_dialog;
@@ -50,15 +62,19 @@ struct _GitgPreferencesDialogPrivate
 	GtkCheckButton *history_show_virtual_stash;
 	GtkCheckButton *history_show_virtual_staged;
 	GtkCheckButton *history_show_virtual_unstaged;
+	GtkCheckButton *history_topo_order;
 	GtkCheckButton *check_button_collapse_inactive;
-	
+	GtkCheckButton *main_layout_vertical;
+
 	GtkCheckButton *check_button_show_right_margin;
 	GtkLabel *label_right_margin;
 	GtkSpinButton *spin_button_right_margin;
-	
+
+	GtkCheckButton *check_button_external_diff;
+
 	GtkEntry *entry_configuration_user_name;
 	GtkEntry *entry_configuration_user_email;
-	
+
 	GtkWidget *table;
 
 	gint prev_value;
@@ -78,9 +94,9 @@ static void
 gitg_preferences_dialog_finalize(GObject *object)
 {
 	GitgPreferencesDialog *dialog = GITG_PREFERENCES_DIALOG (object);
-	
+
 	g_object_unref (dialog->priv->config);
-	
+
 	G_OBJECT_CLASS(gitg_preferences_dialog_parent_class)->finalize(object);
 }
 
@@ -88,7 +104,7 @@ static void
 gitg_preferences_dialog_class_init(GitgPreferencesDialogClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
-	
+
 	object_class->finalize = gitg_preferences_dialog_finalize;
 
 	g_type_class_add_private(object_class, sizeof(GitgPreferencesDialogPrivate));
@@ -98,7 +114,7 @@ static void
 gitg_preferences_dialog_init(GitgPreferencesDialog *self)
 {
 	self->priv = GITG_PREFERENCES_DIALOG_GET_PRIVATE(self);
-	
+
 	self->priv->config = gitg_config_new (NULL);
 }
 
@@ -114,10 +130,10 @@ convert_collapsed(GValue const *source, GValue *dest, gpointer userdata)
 	GitgPreferencesDialog *dialog = GITG_PREFERENCES_DIALOG(userdata);
 
 	gint val = round_val(g_value_get_double(source));
-	
+
 	if (val == dialog->priv->prev_value)
 		return FALSE;
-	
+
 	dialog->priv->prev_value = val;
 	return g_value_transform(source, dest);
 }
@@ -186,6 +202,11 @@ initialize_view(GitgPreferencesDialog *dialog)
 	                             dialog->priv->history_show_virtual_unstaged, 
 	                             "active");
 
+	gitg_data_binding_new_mutual(preferences,
+	                             "history-topo-order",
+	                             dialog->priv->history_topo_order,
+	                             "active");
+
 	gitg_data_binding_new_mutual(preferences,
 	                             "message-show-right-margin",
 	                             dialog->priv->check_button_show_right_margin,
@@ -195,49 +216,64 @@ initialize_view(GitgPreferencesDialog *dialog)
 	                             "message-right-margin-at",
 	                             dialog->priv->spin_button_right_margin,
 	                             "value");
+
+	gitg_data_binding_new_mutual(preferences,
+	                             "main-layout-vertical",
+	                             dialog->priv->main_layout_vertical,
+	                             "active");
+
+	gitg_data_binding_new_mutual(preferences,
+	                             "diff-external",
+	                             dialog->priv->check_button_external_diff,
+	                             "active");
 }
 
 static void
 create_preferences_dialog()
 {
 	GtkBuilder *b = gitg_utils_new_builder("gitg-preferences.ui");
-	
+
 	preferences_dialog = GITG_PREFERENCES_DIALOG(gtk_builder_get_object(b, "dialog_preferences"));
 	g_object_add_weak_pointer(G_OBJECT(preferences_dialog), (gpointer *)&preferences_dialog);
-	
+
 	GitgPreferencesDialogPrivate *priv = preferences_dialog->priv;
-	
+
 	priv->history_search_filter = GTK_CHECK_BUTTON(gtk_builder_get_object(b, "check_button_history_search_filter"));
 	priv->collapse_inactive_lanes = GTK_ADJUSTMENT(gtk_builder_get_object(b, "adjustment_collapse_inactive_lanes"));
-	
+
 	priv->history_show_virtual_stash = GTK_CHECK_BUTTON(gtk_builder_get_object(b, "check_button_history_show_virtual_stash"));
 	priv->history_show_virtual_staged = GTK_CHECK_BUTTON(gtk_builder_get_object(b, "check_button_history_show_virtual_staged"));
 	priv->history_show_virtual_unstaged = GTK_CHECK_BUTTON(gtk_builder_get_object(b, "check_button_history_show_virtual_unstaged"));
-	
+	priv->history_topo_order = GTK_CHECK_BUTTON (gtk_builder_get_object (b, "check_button_history_topo_order"));
+
+	priv->main_layout_vertical = GTK_CHECK_BUTTON (gtk_builder_get_object (b, "check_button_main_layout_vertical"));
+
 	priv->check_button_collapse_inactive = GTK_CHECK_BUTTON(gtk_builder_get_object(b, "check_button_collapse_inactive"));
 	priv->table = GTK_WIDGET(gtk_builder_get_object(b, "table_collapse_inactive_lanes"));
-	
+
 	priv->check_button_show_right_margin = GTK_CHECK_BUTTON(gtk_builder_get_object(b, "check_button_show_right_margin"));
 	priv->label_right_margin = GTK_LABEL(gtk_builder_get_object(b, "label_right_margin"));
 	priv->spin_button_right_margin = GTK_SPIN_BUTTON(gtk_builder_get_object(b, "spin_button_right_margin"));
-	
+
+	priv->check_button_external_diff = GTK_CHECK_BUTTON (gtk_builder_get_object (b, "check_button_external_diff"));
+
 	priv->prev_value = (gint)gtk_adjustment_get_value(priv->collapse_inactive_lanes);
 	g_signal_connect(preferences_dialog, "response", G_CALLBACK(on_response), NULL);
-	
+
 	initialize_view(preferences_dialog);
-	
+
 	priv->entry_configuration_user_name = GTK_ENTRY(gtk_builder_get_object(b, "entry_configuration_user_name"));
 	priv->entry_configuration_user_email = GTK_ENTRY(gtk_builder_get_object(b, "entry_configuration_user_email"));
 
 	gtk_builder_connect_signals(b, preferences_dialog);
 	g_object_unref(b);
-	
+
 	gchar *val;
-	
+
 	val = gitg_config_get_value (priv->config, "user.name");
 	gtk_entry_set_text (priv->entry_configuration_user_name, val ? val : "");
 	g_free (val);
-	
+
 	val = gitg_config_get_value (priv->config, "user.email");
 	gtk_entry_set_text (priv->entry_configuration_user_email, val ? val : "");
 	g_free (val);
@@ -256,29 +292,47 @@ gitg_preferences_dialog_present(GtkWindow *window)
 }
 
 void
-on_collapse_inactive_lanes_changed(GtkAdjustment *adjustment, GParamSpec *spec, GitgPreferencesDialog *dialog)
+on_collapse_inactive_lanes_changed (GtkAdjustment *adjustment,
+                                    GParamSpec *spec,
+                                    GitgPreferencesDialog *dialog)
 {
 	gint val = round_val(gtk_adjustment_get_value(adjustment));
 
 	if (val > 0)
 	{
-		g_signal_handlers_block_by_func(adjustment, G_CALLBACK(on_collapse_inactive_lanes_changed), dialog);
-		gtk_adjustment_set_value(adjustment, val);
-		g_signal_handlers_unblock_by_func(adjustment, G_CALLBACK(on_collapse_inactive_lanes_changed), dialog);
+		g_signal_handlers_block_by_func (adjustment,
+		                                 on_collapse_inactive_lanes_changed,
+		                                 dialog);
+
+		gtk_adjustment_set_value (adjustment, val);
+
+		g_signal_handlers_unblock_by_func (adjustment,
+		                                   on_collapse_inactive_lanes_changed,
+		                                   dialog);
 	}
 }
 
 gboolean
-on_entry_configuration_user_name_focus_out_event(GtkEntry *entry, GdkEventFocus *event, GitgPreferencesDialog *dialog)
+on_entry_configuration_user_name_focus_out_event (GtkEntry *entry,
+                                                  GdkEventFocus *event,
+                                                  GitgPreferencesDialog *dialog)
 {
-	gitg_config_set_value (dialog->priv->config, "user.name", gtk_entry_get_text (entry));
+	gitg_config_set_value (dialog->priv->config,
+	                       "user.name",
+	                       gtk_entry_get_text (entry));
+
 	return FALSE;
 }
 
 gboolean
-on_entry_configuration_user_email_focus_out_event(GtkEntry *entry, GdkEventFocus *event, GitgPreferencesDialog *dialog)
+on_entry_configuration_user_email_focus_out_event (GtkEntry *entry,
+                                                   GdkEventFocus *event,
+                                                   GitgPreferencesDialog *dialog)
 {
-	gitg_config_set_value (dialog->priv->config, "user.email", gtk_entry_get_text (entry));
+	gitg_config_set_value (dialog->priv->config,
+	                       "user.email",
+	                       gtk_entry_get_text (entry));
+
 	return FALSE;
 }
 
